@@ -1,6 +1,6 @@
 from ._serialisation import DefaultSerialiser
-from ._utils import _gen_id, _task_type_for, _is_tuple, _type_name, _annotated_protos_for
-from ._types import RetryPolicy, TaskAlreadyExistsException
+from ._utils import _gen_id, _task_type_for, _is_tuple, _type_name, _annotated_protos_for, _unpack_single_tuple_args
+from ._types import Task, RetryPolicy, TaskAlreadyExistsException
 from ._pipeline import _Pipeline, PipelineBuilder
 from ._context import TaskContext
 from ._builtins import run_pipeline
@@ -147,8 +147,20 @@ class FlinkTasks(object):
             def send(*args, **kwargs):
                 return PipelineBuilder().send(function, *args, **kwargs)
 
-            function.defaults = defaults
+            def to_task(args, kwargs, is_finally=False, parameters={}):
+                parameters = {**defaults(), **parameters}
+
+                if is_finally:
+                    parameters['is_fruitful'] = False
+                
+                module_name = parameters.get('module_name', None)
+                task_type = _task_type_for(function, module_name)
+
+                args = _unpack_single_tuple_args(args)
+                return Task.from_fields(_gen_id(), task_type, args, kwargs, is_finally=is_finally, **parameters)
+
             function.send = send
+            function.to_task = to_task
 
             self.register(function, **defaults())
             return function
@@ -444,7 +456,11 @@ class _FlinkTask(object):
         return task_result, task_exception, pipeline, fn_state
 
     def _to_pipeline_or_task_result(self, task_request, fn_result, fn_state):
-        pipeline, task_result = None, None
+        pipeline, task_result, is_fruitful = None, None, self._is_fruitful
+
+        # allow caller to drop results of this task if it is fruitful
+        if is_fruitful and task_request.HasField('is_fruitful'):
+            is_fruitful = task_request.is_fruitful
 
         # if single result then wrap in tuple as this is the maximal case
         if not _is_tuple(fn_result):
@@ -468,11 +484,11 @@ class _FlinkTask(object):
         if isinstance(fn_result, PipelineBuilder):
             # this new pipeline which once complete will yield the result of the whole pipeline
             # back to the caller as if it were a simple task
-            pipeline = fn_result.to_pipeline(self._serialiser, is_fruitful=self._is_fruitful)
+            pipeline = fn_result.to_pipeline(self._serialiser, is_fruitful=is_fruitful)
             fn_result = ()
 
         # drop the result if the task is marked as not fruitful
-        if not self._is_fruitful:
+        if not is_fruitful:
             fn_result = ()
 
         task_result = _create_task_result(task_request)
