@@ -27,8 +27,7 @@ class Task:
 
     @staticmethod
     def from_fields(task_id, task_type, task_args, task_kwargs, is_finally=None, namespace=None, worker_name=None,
-                    is_fruitful=None, retry_policy=None, module_name=None, with_state=None,
-                    with_context=None):
+                    is_fruitful=None, retry_policy=None, module_name=None, **kwargs):
         proto = TaskEntry(
             task_id=task_id,
             task_type=task_type,
@@ -38,9 +37,7 @@ class Task:
             worker_name=worker_name,
             is_fruitful=is_fruitful,
             retry_policy=retry_policy,
-            module_name=module_name,
-            with_state=with_state,
-            with_context=with_context
+            module_name=module_name
         )
         return Task(proto, task_args, task_kwargs)
 
@@ -83,6 +80,10 @@ class Task:
     def is_fruitful(self):
         return self._proto.is_fruitful
 
+    @is_fruitful.setter
+    def is_fruitful(self, value):
+        self._proto.is_fruitful = value
+
     @property
     def retry_policy(self):
         return self._proto.retry_policy
